import path from 'path'
import webpack from "webpack";

import FriendlyErrorsPlugin from "friendly-errors-webpack-plugin";
import OptimizeCssAssetsPlugin from "optimize-css-assets-webpack-plugin";
import MiniCssExtractPlugin from "mini-css-extract-plugin";
import { VueLoaderPlugin } from "vue-loader";
import TerserPlugin from "terser-webpack-plugin";

import options from "./config/options";
import * as loaders from "./config/loader";

import { searchEntries } from "./utils";

const vue3Dir = path.resolve('./js')

const vue3_entry = searchEntries({
  fileName: "index",
  entryPath: vue3Dir,
  fileNamePrefix: `vue3/js/`,
  isFuzzy: true,
});

// 基础配置
const config = {
  watch: options.__DEV__,
  watchOptions: {
    ignored: /node_modules/,
  },
  mode: process.env.NODE_ENV,
  entry: vue3_entry,
  output: Object.assign(options.output, {
    filename: "[name].js",
  }),
  externals: options.externals,
  resolve: {
    alias: {
      common: path.resolve('../', 'app/Resources/static-src/common'),
      vue: 'vue/dist/vue.esm-browser.js'
    },
    extensions: ["*", ".js", ".jsx", ".vue"],
  },
  module: {
    // noParse: [],
    rules: [
      loaders.vueLoader({
        hotReload: options.__DEV__ || options.__DEBUG__ ? true : false, // 编译时关闭热重载
      }),
      loaders.jsLoader(
        {
          cpuNumber: options.cpuNumber,
        },
        [path.resolve('./', 'node_modules')]
      ),
      loaders.cssLoader({
        minimize: options.__DEV__ || options.__DEBUG__ ? false : true,
        hmr: options.__DEV__,
        reloadAll: true,
      }),
      loaders.lessLoader({
        minimize: options.__DEV__ || options.__DEBUG__ ? false : true,
        hmr: options.__DEV__,
        reloadAll: true,
      }),
    ],
  },
  optimization: {
    minimizer: [new TerserPlugin()],
    splitChunks: {
      cacheGroups: {
        common: {
          name: `vue3/js/${options.commonsChunkFileName}`,
          chunks: "initial", //入口处开始提取代码
          minSize: 300000, //代码最小多大，进行抽离
          minChunks: 6,
        },
      },
    },
  },
  plugins: [
    new MiniCssExtractPlugin({
      filename: "[name].css",
      chunkFilename: "[id].css",
    }),
    new webpack.DefinePlugin({
      // __webpack_public_path__: `window.__publicPath`,
      "process.env": {
        NODE_ENV: JSON.stringify(process.env.NODE_ENV || "development"),
      },
    }),
    // new webpack.ProvidePlugin(options.global),
    new webpack.ContextReplacementPlugin(
      /moment[\\\/]locale$/,
      /^\.\/(zh-cn|en-gb)+\.js$/
    ),
    new VueLoaderPlugin(),
  ],
};


config.plugins = config.plugins.concat(new FriendlyErrorsPlugin());

if (process.env.NODE_ENV === 'production') {
  config.plugins = config.plugins.concat(new OptimizeCssAssetsPlugin());
} else {
  config.devtool = 'source-map';
}

export default config;
